// ====================================================================
// Copyright (c) 1997, 1998 The Apache Group.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer. 
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//
// 3. All advertising materials mentioning features or use of this
//    software must display the following acknowledgment:
//    "This product includes software developed by the Apache Group
//    for use in the Apache HTTP server project (http://www.apache.org/)."
//
// 4. The names "Apache Server" and "Apache Group" must not be used to
//    endorse or promote products derived from this software without
//    prior written permission.
//
// 5. Redistributions of any form whatsoever must retain the following
//    acknowledgment:
//    "This product includes software developed by the Apache Group
//    for use in the Apache HTTP server project (http://www.apache.org/)."
//
// THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
// EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
// ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
// ====================================================================
//
// This software consists of voluntary contributions made by many
// individuals on behalf of the Apache Group and was originally based
// on public domain software written at the National Center for
// Supercomputing Applications, University of Illinois, Urbana-Champaign.
// For more information on the Apache Group and the Apache HTTP server
// project, please see <http://www.apache.org/>.

// JServ - Serve up Java servlets
// by Alexei Kosut <akosut@apache.org>

// Parts are based on examples from  _Java in a Nutshell_ by David Flanagan:
// Written by David Flanagan.  Copyright (c) 1996 O'Reilly & Associates.
// You may study, use, modify, and distribute this example for any purpose.
// This example is provided WITHOUT WARRANTY either expressed or implied.

// ServletMonitor.java:
// - org.apache.jserv.test.ServletMonitor

package gs.admin;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletContext;
import javax.servlet.ServletConfig;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpUtils;

import java.sql.ResultSet;

import gs.db.update;
import gs.db.student;
import gs.db.user;    // remove this later...
import gs.db.groupusers;
import gs.db.RecordHandler;

import gs.html.Html;
import gs.html.Parser;

/**
 */
public class students extends HttpServlet {

    /**
     * Service the request.
     *
     * @exception  ServletException if there is any sort of Exception
     *
     */

  public static int MIN_SSN_LENGTH = 9;
  public static int MIN_PASSWORD_LENGTH = 5;

  private final String DEFAULT_USER_GROUP = "students";
  
  public void doPost
  (
   HttpServletRequest req,
   HttpServletResponse response
    )
       throws IOException, ServletException
  {
    
    response.setStatus(HttpServletResponse.SC_OK); 
    response.setContentType("text/html");
    
    ServletOutputStream out = response.getOutputStream();
    
    out.println( Html.header("Student Addition") );
    
    String command   = req.getParameter("comm");
    String ssn       = req.getParameter("ssn1")
                       +req.getParameter("ssn2")
                       +req.getParameter("ssn3"); 
    String firstname = req.getParameter("firstname"); 
    String midname   = req.getParameter("midname");
    String lastname  = req.getParameter("lastname"); 
    String gspwd     = req.getParameter("gspwd"); 
    String confirm   = req.getParameter("confirmgspwd"); 

    if(ssn.equals("") || firstname.equals("") || lastname.equals("") 
       || gspwd.equals("") || confirm.equals("")) 
      out.println("<h2>ERROR: Please enter in all the fields in red!");
    else if (ssn.length() != students.MIN_SSN_LENGTH) 
      out.println("<h2>ERROR: Social security number must be 9 digits long!");
    else if (!gspwd.equals(confirm)) 
      out.println("<h2>ERROR: Passwords do not match!");
    else if (gspwd.length() < students.MIN_PASSWORD_LENGTH) 
      out.println("<h2>ERROR: Password must be at least 5 characters long!");
    else {
      String student_id= req.getParameter("student_id"); 
      String email     = req.getParameter("email"); 
      String dept      = req.getParameter("dept"); 
      String year      = req.getParameter("year"); 
      String login     = req.getParameter("login"); 
      String unixpwd   = "";
      String url       = req.getParameter("url");  

      student s = new student(ssn, gspwd, firstname, midname, lastname,
			      student_id, email, dept, year, login, unixpwd, url);  
      groupusers gu = new groupusers(DEFAULT_USER_GROUP, ssn);  
      
      try {
	String templateFile = req.getPathTranslated();
	Hashtable ht = new Hashtable();

	if( command.equals("new") ) {
	  update.updateData( s.insertSQL() );
	  update.updateData( gu.insertSQL() );
	  ht.put("message", "Student addition comleted successfully!");
	} 
	else {
	  update.updateData( s.updateSQL() );
	  ht.put("message", "Student information update comleted successfully!");
	}

	Parser.parseHtml
	  (templateFile,
	   ht,
	   out);
	
      } catch( Exception e ) {
	out.println("<h2>ERROR: "+e.getMessage());
      }   
      
      out.println( Html.footer() );
    }
  }
  
  public void doGet(HttpServletRequest req, HttpServletResponse response)
       throws IOException, ServletException
  {
    
    response.setStatus(HttpServletResponse.SC_OK); 
    response.setContentType("text/html");
    
    ServletOutputStream out = response.getOutputStream();
    
    out.println( Html.header("Student Records") );
    
    String command= req.getParameter("comm"); 
    if(command.equals("list"))
      listStudents(out);
      
    else if(command.equals("browse"))
      browseStudent(out, req.getPathTranslated(), req.getParameter("user"));

    else if(command.equals("new"))
      newStudent(out, req.getPathTranslated());

    else if(command.equals("delete"))
      deleteStudent(out, req.getPathTranslated(), 
		    (String)(req.getParameter("user")), 
		    (String)(req.getParameter("answer")));

    out.println( Html.footer() );
    out.close();
  }
  
  void listStudents(ServletOutputStream out)
       throws IOException
  {
    class studentParser extends RecordParser {
      

    try {
      out.println("<BASE TARGET=info>");

      try {
	Hashtable ht = new Hashtable();

	StringBuffer output = new StringBuffer();
	student.students("ssn,firstname,midname,lastname","","lastname", 
	 new RecordHandler() {
	  public void handleRecord(ResultSet rs, Hashtable ht) 
	    throws java.sql.SQLException 
	    {   
	      ht.get("firstname")
	    String f= rs.getString("firstname");         
	    String m= rs.getString("midname");
	    String l= rs.getString("lastname");
	    String s= rs.getString("ssn");

	    output.append("<tr><td><a href=\"students/Grading/students/browse.html?comm=browse&user="+
			s+"\">"+l+", "+f+" "+m+"</a></td></tr>\n");
	}
      }, output);




      out.println("<a href=\"students/Grading/students/update.html?comm=new\">Add Student</a></p>");      
      out.println("<p><table border=1><tr><th>Students</th></tr>");
      out.println(output.toString()+"</table>");

    } catch( Exception e ) {
      out.println("<h2>ERROR: "+e.getMessage()+"</h2>");
    }   
  }

  void browseStudent(ServletOutputStream out, String templateFile, String ssn) 
       throws IOException
  {
    if(ssn.length() >= students.MIN_SSN_LENGTH) {
      try {
	Hashtable ht = student.getFields(ssn);
	ht.put("ssn1", ssn.substring(0,3));
	ht.put("ssn2", ssn.substring(3,5));
	ht.put("ssn3", ssn.substring(5));
	Parser.parseHtml
	  (templateFile,
	   ht,
	   out);

      } catch(Exception e) {
	out.println("<h2>ERROR:"+e.getMessage()+"</h2>");
      }
    } else {
      out.println("<h2>ERROR:Illegal SSN '"+ssn+"'.</h2>");
    }
  }

  void newStudent
  (
   ServletOutputStream out, 
   String templateFile
    )
       throws IOException
  {
    try {
      Hashtable ht = student.getEmptyFields();
      ht.put("ssn1", "");
      ht.put("ssn2", "");
      ht.put("ssn3", "");
      ht.put("year0", "");
      ht.put("year1", "");
      ht.put("year2", "");
      ht.put("year3", "");
      ht.put("year4", "");
      ht.put("year5", "SELECTED");
      ht.put("year6", "");
      ht.put("year7", "");
      ht.put("comm", "new");
      
      Parser.parseHtml
	(templateFile,
	 ht,
	 out);
      
    } catch(Exception e) {
      out.println("<h2>ERROR:"+e.getMessage()+"</h2>");
    }
  }

  void deleteStudent
  (
   ServletOutputStream out, 
   String templateFile, 
   String ssn,
   String answer
    ) 
       throws IOException
  {
    try {
      Hashtable ht = new Hashtable();
      if(answer == null) 
	{
	  ht.put("message", "Do you really want to delete the student <i>"+ssn+"</i>?");
	  ht.put("actionurl", "message.html");
	  ht.put("comm","delete");
	  ht.put("user",ssn);
	} 
      else if(answer.equals("YES")) 
	{
	  student s = new student();
	  groupusers gu = new groupusers( "", ssn );
	  
	  s.ssn = ssn;
	  update.updateData( s.deleteSQL() );
	  update.updateData( gu.deleteSQL() );
	  ht.put("message", "User information has been succesfully deleted!");
	}
      Parser.parseHtml
	(templateFile,
	 ht,
	 out);
      
    } catch(Exception e) {
      out.println("<h2>ERROR:"+e.getMessage()+"</h2>");
    }
  }
}
